@@ -710,7 +710,10 @@ def _preprocess_faces(text, split_object=False):
 def export_obj(mesh,
                include_normals=True,
                include_color=True,
-               include_texture=False,
+               include_texture=True,
+               return_texture=False,
+               write_texture=True,
+               resolver=None,
                digits=8):
     """
     Export a mesh as a Wavefront OBJ file.
@@ -725,14 +728,17 @@ def export_obj(mesh,
     include_color : bool
       Include vertex color in export
     include_texture bool
-      Include texture in export?
-      False by default as it will change the return
-      values to include files that must be saved in the
-      same location as the exported mesh.
+      Include `vt` texture in file text
+    return_texture : bool
+      If True, return a dict with texture files
+    write_texture : bool
+      If True and a writable resolver is passed
+      write the referenced texture files with resolver
+    resolver : None or trimesh.resolvers.Resolver
+      Resolver which can write referenced text objects
     digits : int
       Number of digits to include for floating point
 
-
     Returns
     -----------
     export : str
@@ -831,8 +837,14 @@ def export_obj(mesh,
     objects.appendleft('# https://github.com/mikedh/trimesh')
     # combine elements into a single string
     text = '\n'.join(objects)
+
+    # if we have a resolver and have asked to write texture
+    if write_texture and resolver is not None and tex_data is not None:
+        # not all resolvers have a write method
+        [resolver.write(k, v) for k, v in tex_data.items()]
+
     # if we exported texture it changes returned values
-    if include_texture and tex_data is not None:
+    if return_texture:
         return text, tex_data
 
     return text
